﻿using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Algorithms.MOCMAEvolutionStrategy {
  internal static class Utilities {
    internal static double[][] ToArray(DoubleMatrix m) {
      if (m == null) return null;
      var i = m.Rows - 1;
      var a = new double[i][];
      for (i--; i >= 0; i--) {
        var j = m.Columns;
        a[i] = new double[j];
        for (j--; j >= 0; j--) a[i][j] = m[i, j];
      }
      return a;
    }

    public static int ArgMin<T>(IEnumerable<T> values, IComparer<T> comparer) {
      var mindex = -1;
      var min = default(T);
      var i = 0;
      foreach (var v in values) {
        if (mindex < 0 || comparer.Compare(v, min) < 0) {
          min = v;
          mindex = i;
        }
        i++;
      }
      return mindex;

    }
    public static int ArgMax<T>(IEnumerable<T> values, IComparer<T> comparer) {
      var mindex = -1;
      var min = default(T);
      var i = 0;
      foreach (var v in values) {
        if (mindex < 0 || comparer.Compare(v, min) > 0) {
          min = v;
          mindex = i;
        }
        i++;
      }
      return mindex;
    }
    private class DoubleComparer : IComparer<double> {
      public int Compare(double x, double y) {
        return x.CompareTo(y);
      }
    }
    public static int ArgMax(this IEnumerable<double> values) {
      return ArgMax(values, new DoubleComparer());
    }
    public static int ArgMin(this IEnumerable<double> values) {
      return ArgMin(values, new DoubleComparer());
    }
  }
}
